@@ -30,15 +30,19 @@ Unit tests for eb command line options.
 
 import os
 import re
+import shutil
 import sys
 import tempfile
 from unittest import TestCase, TestLoader
 from unittest import main as unittestmain
 
+import easybuild.tools.options as eboptions
 from easybuild.main import main
 from easybuild.framework.easyconfig import BUILD, CUSTOM, DEPENDENCIES, EXTENSIONS, FILEMANAGEMENT, LICENSE
 from easybuild.framework.easyconfig import MANDATORY, MODULES, OTHER, TOOLCHAIN
+from easybuild.tools import config
 from easybuild.tools.filetools import read_file, write_file
+from easybuild.tools.modules import modules_tool
 from easybuild.tools.options import EasyBuildOptions
 from vsc import fancylogger
 
@@ -46,9 +50,13 @@ class CommandLineOptionsTest(TestCase):
     """Testcases for command line options."""
 
     logfile = None
+    # initialize configuration so modules_tool() function works
+    eb_go = eboptions.parse_options()
+    config.init(eb_go.options, eb_go.get_options_by_section('config'))
 
     def setUp(self):
         """Prepare for running unit tests."""
+        self.pwd = os.getcwd()
         # create log file
         fd, self.logfile = tempfile.mkstemp(suffix='.log', prefix='eb-options-test-')
         os.close(fd)
@@ -57,6 +65,7 @@ class CommandLineOptionsTest(TestCase):
         """Post-test cleanup."""
         # removing of self.logfile can't be done here, because it breaks logging
         os.remove(self.logfile)
+        os.chdir(self.pwd)
 
     def test_help_short(self, txt=None):
         """Test short help message."""
@@ -101,7 +110,7 @@ class CommandLineOptionsTest(TestCase):
         """Test using no arguments."""
 
         try:
-            main(([], self.logfile))
+            main(([], self.logfile, False))
         except (SystemExit, Exception), err:
             pass
         outtxt = read_file(self.logfile)
@@ -119,7 +128,7 @@ class CommandLineOptionsTest(TestCase):
                     debug_arg,
                    ]
             try:
-                main((args, self.logfile))
+                main((args, self.logfile, False))
             except (SystemExit, Exception), err:
                 myerr = err
             outtxt = read_file(self.logfile)
@@ -138,7 +147,7 @@ class CommandLineOptionsTest(TestCase):
                    ]
             myerr = None
             try:
-                main((args, self.logfile))
+                main((args, self.logfile, False))
             except (SystemExit, Exception), err:
                 myerr = err
             outtxt = read_file(self.logfile)
@@ -160,7 +169,7 @@ class CommandLineOptionsTest(TestCase):
                     quiet_arg,
                    ]
             try:
-                main((args, self.logfile))
+                main((args, self.logfile, False))
             except (SystemExit, Exception), err:
                 pass
             outtxt = read_file(self.logfile)
@@ -186,11 +195,12 @@ class CommandLineOptionsTest(TestCase):
         # check log message without --force
         args = [
                 eb_file,
+                '--debug',
                ]
 
         error_thrown = False
         try:
-            main((args, self.logfile))
+            main((args, self.logfile, False))
         except (SystemExit, Exception), err:
             error_thrown = err
 
@@ -198,7 +208,7 @@ class CommandLineOptionsTest(TestCase):
 
         self.assertTrue(not error_thrown, "No error is thrown if software is already installed (error_thrown: %s)" % error_thrown)
 
-        already_msg = "GCC \(version 4.6.3\) is already installed"
+        already_msg = "GCC/4.6.3 is already installed"
         self.assertTrue(re.search(already_msg, outtxt), "Already installed message without --force, outtxt: %s" % outtxt)
 
         # clear log file
@@ -208,9 +218,10 @@ class CommandLineOptionsTest(TestCase):
         args = [
                 eb_file,
                 '--force',
+                '--debug',
                ]
         try:
-            main((args, self.logfile))
+            main((args, self.logfile, False))
         except (SystemExit, Exception), err:
             pass
         outtxt = read_file(self.logfile)
@@ -223,6 +234,83 @@ class CommandLineOptionsTest(TestCase):
         else:
             os.environ.pop('MODULEPATH')
 
+    def test_skip(self):
+        """Test skipping installation of module (--skip, -k)."""
+
+        # use temporary paths for build/install paths, make sure sources can be found
+        buildpath = tempfile.mkdtemp()
+        installpath = tempfile.mkdtemp()
+        sourcepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sandbox', 'sources')
+
+        # set MODULEPATH to included modules
+        orig_modulepath = os.getenv('MODULEPATH', None)
+        os.environ['MODULEPATH'] = os.path.abspath(os.path.join(os.path.dirname(__file__), 'modules'))
+
+        # use toy-0.0.eb easyconfig file that comes with the tests
+        eb_file = os.path.join(os.path.dirname(__file__), 'easyconfigs', 'toy-0.0.eb')
+
+        # check log message with --skip for existing module
+        args = [
+                eb_file,
+                '--sourcepath=%s' % sourcepath,
+                '--buildpath=%s' % buildpath,
+                '--installpath=%s' % installpath,
+                '--force',
+                '--skip',
+                '--debug',
+               ]
+
+        try:
+            main((args, self.logfile, True))
+        except (SystemExit, Exception), err:
+            pass
+
+        outtxt = read_file(self.logfile)
+
+        found_msg = "Module toy/0.0 found.\n[^\n]+Going to skip actual main build"
+        found = re.search(found_msg, outtxt, re.M)
+        self.assertTrue(found, "Module found message present with --skip, outtxt: %s" % outtxt)
+
+        # cleanup for next test
+        write_file(self.logfile, '')
+        os.chdir(self.pwd)
+        modules_tool().purge()
+
+        # check log message with --skip for non-existing module
+        args = [
+                eb_file,
+                '--sourcepath=%s' % sourcepath,
+                '--buildpath=%s' % buildpath,
+                '--installpath=%s' % installpath,
+                '--try-software-version=1.2.3.4.5.6.7.8.9',
+                '--try-amend=sources=toy-0.0.tar.gz,toy-0.0.tar.gz',  # hackish, but fine
+                '--force',
+                '--skip',
+                '--debug',
+               ]
+        try:
+            main((args, self.logfile, True))
+        except (SystemExit, Exception), err:
+            pass
+        outtxt = read_file(self.logfile)
+
+        found = re.search(found_msg, outtxt)
+        self.assertTrue(not found, "Module found message not there with --skip for non-existing modules: %s" % outtxt)
+
+        not_found_msg = "No module toy/1.2.3.4.5.6.7.8.9 found. Not skipping anything."
+        not_found = re.search(not_found_msg, outtxt)
+        self.assertTrue(not_found, "Module not found message there with --skip for non-existing modules: %s" % outtxt)
+
+        # restore original MODULEPATH
+        if orig_modulepath is not None:
+            os.environ['MODULEPATH'] = orig_modulepath
+        else:
+            os.environ.pop('MODULEPATH')
+
+        # cleanup
+        shutil.rmtree(buildpath)
+        shutil.rmtree(installpath)
+
     def test_job(self):
         """Test submitting build as a job."""
 
@@ -247,7 +335,7 @@ class CommandLineOptionsTest(TestCase):
                     '--job',
                    ] + job_args
             try:
-                main((args, self.logfile))
+                main((args, self.logfile, False))
             except (SystemExit, Exception), err:
                 pass
             outtxt = read_file(self.logfile)
@@ -286,7 +374,7 @@ class CommandLineOptionsTest(TestCase):
                     stdout_arg,
                    ]
             try:
-                main((args, dummylogfn))
+                main((args, dummylogfn, False))
             except (SystemExit, Exception), err:
                 myerr = err
 
@@ -331,7 +419,7 @@ class CommandLineOptionsTest(TestCase):
                     args.extend(['-e', custom])
 
                 try:
-                    main((args, dummylogfn))
+                    main((args, dummylogfn, False))
                 except (SystemExit, Exception), err:
                     pass
                 outtxt = read_file(self.logfile)
@@ -359,15 +447,16 @@ class CommandLineOptionsTest(TestCase):
                 os.remove(dummylogfn)
 
         # also check whether available custom easyconfig parameters are listed
-        orig_sys_path = sys.path
+        orig_sys_path = sys.path[:]
 
         import easybuild
-        sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'easyblocks_sandbox')))
+        sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'sandbox')))
         easybuild = reload(easybuild)
         import easybuild.easyblocks
-        easybuild.easyblocks = reload(easybuild.easyblocks)
+        reload(easybuild.easyblocks)
         import easybuild.easyblocks.generic
-        easybuild.easyblocks.generic = reload(easybuild.easyblocks.generic)
+        reload(easybuild.easyblocks.generic)
+        reload(easybuild.tools.module_naming_scheme)  # required to run options unit tests stand-alone
 
         run_test(custom='EB_foo', extra_params=['foo_extra1', 'foo_extra2'])
         run_test(custom='bar', extra_params=['bar_extra1', 'bar_extra2'])
@@ -389,7 +478,7 @@ class CommandLineOptionsTest(TestCase):
                 '--unittest-file=%s' % self.logfile,
                ]
         try:
-            main((args, dummylogfn))
+            main((args, dummylogfn, False))
         except (SystemExit, Exception), err:
             pass
         outtxt = read_file(self.logfile)
@@ -406,6 +495,40 @@ class CommandLineOptionsTest(TestCase):
         if os.path.exists(dummylogfn):
             os.remove(dummylogfn)
 
+    def test_avail_lists(self):
+        """Test listing available values of certain types."""
+
+        fd, dummylogfn = tempfile.mkstemp(prefix='easybuild-dummy', suffix='.log')
+        os.close(fd)
+
+        name_items = {
+            'modules-tools': ['EnvironmentModulesC', 'Lmod'],
+            'module-naming-schemes': ['EasyBuildModuleNamingScheme'],
+        }
+        for (name, items) in name_items.items():
+            args = [
+                    '--avail-%s' % name,
+                    '--unittest-file=%s' % self.logfile,
+                   ]
+            try:
+                main((args, dummylogfn, False))
+            except (SystemExit, Exception), err:
+                pass
+            outtxt = read_file(self.logfile)
+
+            words = name.replace('-', ' ')
+            info_msg = r"INFO List of supported %s:" % words
+            self.assertTrue(re.search(info_msg, outtxt), "Info message with list of available %s" % words)
+            for item in items:
+                res = re.findall("^\s*%s" % item, outtxt, re.M)
+                self.assertTrue(res, "%s is included in list of available %s" % (item, words))
+                # every item should only be mentioned once
+                n = len(res)
+                self.assertEqual(n, 1, "%s is only mentioned once (count: %d)" % (item, n))
+
+        if os.path.exists(dummylogfn):
+            os.remove(dummylogfn)
+
     def test_list_easyblocks(self):
         """Test listing easyblock hierarchy."""
 
@@ -413,13 +536,14 @@ class CommandLineOptionsTest(TestCase):
         os.close(fd)
 
         # adjust PYTHONPATH such that test easyblocks are found
-        orig_sys_path = sys.path
+        orig_sys_path = sys.path[:]
 
         import easybuild
-        sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'easyblocks_sandbox')))
+        sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'sandbox')))
         easybuild = reload(easybuild)
         import easybuild.easyblocks
-        easybuild.easyblocks = reload(easybuild.easyblocks)
+        reload(easybuild.easyblocks)
+        reload(easybuild.tools.module_naming_scheme)  # required to run options unit tests stand-alone
 
         # simple view
         for list_arg in ['--list-easyblocks', '--list-easyblocks=simple']:
@@ -432,7 +556,7 @@ class CommandLineOptionsTest(TestCase):
                     '--unittest-file=%s' % self.logfile,
                    ]
             try:
-                main((args, dummylogfn))
+                main((args, dummylogfn, False))
             except (SystemExit, Exception), err:
                 pass
             outtxt = read_file(self.logfile)
@@ -454,7 +578,7 @@ class CommandLineOptionsTest(TestCase):
                 '--unittest-file=%s' % self.logfile,
                ]
         try:
-            main((args, dummylogfn))
+            main((args, dummylogfn, False))
         except (SystemExit, Exception), err:
             pass
         outtxt = read_file(self.logfile)
@@ -485,7 +609,7 @@ class CommandLineOptionsTest(TestCase):
                 '--unittest-file=%s' % self.logfile,
                ]
         try:
-            main((args, dummylogfn))
+            main((args, dummylogfn, False))
         except (SystemExit, Exception), err:
             pass
         outtxt = open(self.logfile, 'r').read()
@@ -508,7 +632,7 @@ class CommandLineOptionsTest(TestCase):
                ]
         myerr = None
         try:
-            main((args, self.logfile))
+            main((args, self.logfile, False))
         except (SystemExit, Exception), err:
             myerr = err
         outtxt = read_file(self.logfile)
